#ifndef _MYSQL_SQL_SYNTAX_CHECK_H_
#define _MYSQL_SQL_SYNTAX_CHECK_H_


#include "mysql_sql_parser_base.h"
#include "grtsqlparser/sql_syntax_check.h"


/** Implements DBMS specifics.
 * 
 * @ingroup sqlparser
 */
class MYSQL_SQL_PARSER_PUBLIC_FUNC Mysql_sql_syntax_check
  : protected Mysql_sql_parser_base, public Sql_syntax_check
{
public:
  Mysql_sql_syntax_check();

  int check_sql_syntax(const std::string &sql);
  int check_trigger_syntax(const std::string &sql);
  int check_view_syntax(const std::string &sql);
  int check_routine_syntax(const std::string &sql);

protected:
  typedef sigc::slot<Parse_result, const MyxSQLTreeItem *> Check_sql_statement_syntax;

  Check_sql_statement_syntax _check_sql_statement_syntax;

  int process_sql_statement(const MyxSQLTreeItem *tree);
  int check_sql_statement_syntax(const std::string &sql, Check_sql_statement_syntax check_sql_statement_syntax);

  Parse_result do_check_sql_syntax(const MyxSQLTreeItem *tree);
  Parse_result do_check_trigger_syntax(const MyxSQLTreeItem *tree);
  Parse_result do_check_view_syntax(const MyxSQLTreeItem *tree);
  Parse_result do_check_routine_syntax(const MyxSQLTreeItem *tree);

  bool _use_delimiter;

  class Null_state_keeper : Mysql_sql_parser_base::Null_state_keeper
  {
  public:
    Null_state_keeper(Mysql_sql_syntax_check *sql_parser)
      : Mysql_sql_parser_base::Null_state_keeper(sql_parser), _sql_parser(sql_parser) {}
    ~Null_state_keeper();
  private:
    Mysql_sql_syntax_check *_sql_parser;
  };
};


#endif // _MYSQL_SQL_SYNTAX_CHECK_H_
